<?php
//
// Copyright (c) 2011-2018 Nagios Enterprises, LLC. All rights reserved.
//

require_once(dirname(__FILE__) . '/../includes/common.inc.php');
require_once(dirname(__FILE__) . '/../includes/components/nxti/includes/utils-traps.inc.php');


// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables and check pre-reqs
grab_request_vars();
check_prereqs();
check_authentication();


// Only admins can access this page
if (!is_admin()) {
    echo _("You are not authorized to access this feature. Contact your system administrator for more information, or to obtain access to this feature.");
    exit();
}


route_request();

function route_request()
{
    global $request;

    if (isset($request["download"])) {
        // floppy
        do_download();
    } 
    else if (isset($request["upload"])) {
        // Blue button
        $processMib = false;
        $import = false;
        if (isset($request["processMibCheck"])) {
            if ($request["processMibCheck"] === 'process') {
                // box checked
                $processMib = true;
            }
            else if ($request['processMibCheck'] === 'nxti') {
                if (enterprise_features_enabled()) {
                    $import = true;
                }
                else {
                    show_mibs(true, _("Upload failed: the SNMP Trap Interface is an enterprise-only feature."));
                }
            }
        }
        do_upload($processMib, $import);
    } 
    else if (isset($request["delete"])) {
        // red x
        do_delete();
    }
    else if (isset($request['process-all'])) {
        // "Process All Traps"
        do_process_all();
    }
    else if (isset($request['nxti'])) {
        // "Process All and Import to Trap Interface" or Import Action.
        if (enterprise_features_enabled()) {
            do_nxti_import();
        }
        else {
            show_mibs(true, _("The SNMP Trap Interface is an enterprise-only feature."));
        }
    }
    else {
        show_mibs();
    }

    exit;
}


/**
 * @param bool   $error
 * @param string $msg
 */
function show_mibs($error = false, $msg = "")
{
    $mibs = get_mibs();

    do_page_start(array("page_title" => _('Manage MIBs')), true);

    $users = array();
    $u = explode("\n", file_get_contents('/etc/passwd'));
    foreach ($u as $l) {
        if (!empty($l)) {
            $x = explode(":", $l);
            $users[$x[2]] = $x[0];
        }
    }

    $groups = array();
    $g = explode("\n", file_get_contents('/etc/group'));
    foreach ($g as $l) {
        if (!empty($l)) {
            $x = explode(":", $l);
            $groups[$x[2]] = $x[0];
        }
    }
?>

    <h1><?php echo _('Manage MIBs'); ?></h1>

    <?php display_message($error, false, $msg); ?>

    <p>
        <?php echo _('Manage the MIBs installed on this server in') . ' <b>' . get_mib_dir() .  '</b>. '.  _('There are hundreds of MIBs available at'); ?>
        <a href="http://www.mibdepot.com/" target="_blank" rel="noreferrer">mibdepot<i class="fa fa-external-link fa-ml"></i></a> <?php echo _('and'); ?> <a href="http://www.oidview.com/mibs/detail.html" target="_blank">oidview<i class="fa fa-external-link fa-ml"></i></a>.
    </p>

    <div class="well">
        <form enctype="multipart/form-data" action="" method="post" style="margin: 0;">
            <?php echo get_nagios_session_protector(); ?>
            <input type="hidden" name="MAX_FILE_SIZE" value="<?php echo get_php_upload_max_filesize(); ?>">

            <div class="fl" style="height: 29px; line-height: 29px; margin-right: 10px; font-weight: bold; color: #666;">
                <label><?php echo _('Upload a MIB'); ?>:</label>
            </div>
            <div class="fl">
                <div class="input-group" style="width: 240px;">
                    <span class="input-group-btn">
                        <span class="btn btn-sm btn-default btn-file">
                            <?php echo _('Browse'); ?>&hellip; <input type="file" name="uploadedfile">
                        </span>
                    </span>
                    <input type="text" class="form-control" style="width: 200px;" readonly>
                </div>
            </div>
            <div class="fl" style="margin: 0 15px;">
                <div class="radio" style="margin: -15px 10px;">
                    <table>
                        <tr>
                            <td>
                                <label>
                                    <input style="display: block" type="radio" id="upload-do-nothing" name="processMibCheck" value="nothing">
                                    <?php echo _("Upload only"); ?>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <label>
                                    <input style="display: block" type="radio" id="upload-do-process" name="processMibCheck" value="process">
                                    <?php echo _("Add to SNMPTT"); ?>
                                </label>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <label>
                                    <input style="display: block" type="radio" id="upload-do-nxti" name="processMibCheck" value="nxti" checked>
                                    <?php echo _("Add to SNMP Trap Interface"); ?>
                                </label>
                            </td>
                        </tr>
                    </table>
                </div>
            </div>
            <button type="submit" name="upload" class="btn btn-sm btn-primary"><?php echo _('Upload MIB'); ?></button>
            <div class="fr">
                <button type="submit" name="process-all" class="btn btn-sm btn-default"><?php echo _("Process All Traps");?></button>
                <button type="submit" name="nxti" class="btn btn-sm btn-default"><?php echo _("Import All to Trap Interface");?></button>
            </div>
            <div class="clear"></div>
        </form>
    </div>

    <table class="table table-condensed table-striped table-bordered table-auto-width">
        <thead>
            <tr>
                <th><?php echo _('MIB'); ?></th>
                <th><?php echo _('File'); ?></th>
                <th><?php echo _('Owner'); ?></th>
                <th><?php echo _('Group'); ?></th>
                <th><?php echo _('Permissions'); ?></th>
                <th><?php echo _('Date'); ?></th>
                <th class="center"><?php echo _('Actions'); ?></th>
            </tr>
        </thead>
        <tbody>

        <?php
        // case insensitive sorting
        usort($mibs, function($x, $y) {
           return strcasecmp($x['mibname'] , $y['mibname']);
        });
        
        $x = 0;
        foreach ($mibs as $mib) {

            if (array_key_exists($mib['group'], $groups)) {
                $group = $groups[$mib["group"]];
                if ($group != 'nagios' && $group != 'nagcmd') {
                    $group = '<em>' . $group . '</em>';
                }
            } else {
                $group = $mib['group'];
            }

            if (array_key_exists($mib['owner'], $users)) {
                $user = $users[$mib["owner"]];
                if ($user != 'nagios' && $user != 'nagcmd') {
                    $user = '<em>' . $user . '</em>';
                }
            } else {
                $user = $mib['owner'];
            }

            echo "<tr>";
            echo "<td>" . $mib["mibname"] . "</td>";
            echo "<td>" . $mib["file"] . "</td>";
            echo "<td>" . $user . "</td>";
            echo "<td>" . $group . "</td>";
            echo "<td>" . $mib["permstring"] . "</td>";
            echo "<td>" . $mib["date"] . "</td>";
            echo '<td class="center">';
            echo "<a href='?download=" . $mib["file"] . "'><img src='" . theme_image("download.png") . "' class='tt-bind' alt='" . _('Download') . "' title='" . _('Download') . "'></a> ";
            echo "<a href='?delete=" . $mib["file"] . "&nsp=" . get_nagios_session_protector_id() . "'><img src='" . theme_image("cross.png") . "' class='tt-bind' alt='" . _('Delete') . "' title='" . _('Delete') . "'></a>";
            echo "</td>";
            echo "</tr>\n";
        }
        ?>

        </tbody>
    </table>

    <?php

    do_page_end(true);
    exit();
}

/* Given a list of event names to comment out in snmptt.conf,
 * parse through the file line by line and comment out the entire
 * event def'n.
 */
function remove_from_snmptt_conf($event_names) {
    $the_file = '/etc/snmp/snmptt.conf';
    $lines = file($the_file);
    $commenting = false;

    foreach ($lines as $index => $line) {
        if (preg_match('/^\s*?EVENT.*$/', $line)) {
            // new event defn, figure out whether to keep commenting.
            $terms = explode(' ', $line);
            $this_event = $terms[1];
            $commenting = false;

            if (in_array($this_event, $event_names)) {
                $lines[$index] = "# This trap definition is being managed by NXTI\n#" . $line;
                $commenting = true;
            }
        }
        else {
            if ($commenting) {
                $lines[$index] = "#" . $line;
            }
        }
    }

    $lines = implode('', $lines);
    $success = file_put_contents($the_file, $lines);
    return $success;
}

// Uses nxti_import.php alongside do_process_all() to move MIB  
// definitions into the Trap Interface.
function do_nxti_import() {

    // demo mode
    if (in_demo_mode() == true) {
        show_mibs(true, _("Changes are disabled while in demo mode."));
    }

    // check session
    check_nagios_session_protector();

    $file = grab_request_var("nxti", "");

    if (!empty($file)) {
        // Specific "Import" Action was clicked - 
        $file = str_replace("..", "", $file);
        $file = str_replace("/", "", $file);
        $file = str_replace("\\", "", $file);

        $dir = get_mib_dir();
        $conf_files = $dir . "/" . $file;
    }
    else {
        // Process All and Import was clicked.
        $conf_files = do_process_all(true);
    }
    $message = "";

    $result_okay = handle_nxti_import($conf_files, $message);
    show_mibs(!$result_okay, $message);
}

function handle_nxti_import($files ,&$message) {
    if (!is_array($files)) {
        $files = array($files);
    }

    $base_command = '../../scripts/nxti_import.php '; // Have nxti_import print event names, \n-delimited
    $okay = true;
    $message .= sprintf(_("Note: If a conf file is missing from the list, it may not contain any trap definitions.%s"), '<br/>');

    $all_events = array(); // To be removed from snmptt.conf
    foreach ($files as $ind => $file) {
        $command = $base_command . escapeshellarg($file);
        $rc = 0;
        $event_names = array();
        $unused = exec($command, $event_names, $rc);

        $all_events = array_merge($all_events, $event_names);
        if ($rc !== 0) {
            $okay = false;
            $message .= sprintf(_("Failed to import conf file %s %s"), $files[$ind], '<br/>');
            unset($files[$ind]);
        }
        else {
            $files[$ind] = basename($file);
            $message .= sprintf(_("Imported conf file %s %s"), $files[$ind], '<br/>');
        }
    }

    $db_message = '';
    $error = commit_trapdata_db($db_message);
    if (!$error) {
        $message .= $db_message . '<br/>';
    }

    // Removes definitions added by addmib
    remove_from_snmptt_conf($all_events);

    // From here to end of fn removes definitions added by snmpttconvertmib.
    // Poor man's sed
    // Look for the part where we read in conf files, remove any which are now managed by NXTI.
    $ini = '/etc/snmp/snmptt.ini';
    $lines = file($ini);

    $started = false;
    foreach ($lines as $index => $line) {
        // Trigger once
        if (!$started && substr(trim($line), 0, 17) !== "snmptt_conf_files") {
            continue;
        }

        $started = true;
        // If we match any of the base filenames
        if (preg_match('/'.implode('|', $files).'/i', $line)) {
            // This is managed by NXTI, so we can remove the 'manual' conf file.
            unset($lines[$index]);
        }
        if (trim($lines[$index+1]) === "END") {
            break;
        }
    }

    $lines = implode('', $lines);
    $result = file_put_contents($ini, $lines);

    $restart_message = '';
    restart_snmptt($restart_message);
    $message .= $restart_message . "<br/>";

    return $okay;
}

// Same as ticking the checkbox on an uploaded file, but does it for every file that's already been uploaded.
function do_process_all($nxti_call = false) {
    $source_directory = get_mib_dir();
    $dest_directory = get_processing_destination($nxti_call);

    $success_count = 0;
    $total_count = 0;
    $source_directory_fd = opendir($source_directory);
    if ($nxti_call) {
        $files = array();
    }
    while (($source_file = readdir($source_directory_fd)) !== false) {
        $source_file = $source_directory . '/' . $source_file;
        if (is_dir($source_file)) // This is not a recursive search
            continue;
        $total_count += 1;
        if (run_addmib($source_file, $message) != 0) {
            // Setup convert_mib
            $target_path_parts = pathinfo($source_file);
            $dest_file = $dest_directory . '/' . $target_path_parts['filename'] . '.txt';
            unlink($dest_file); // snmpttmibconvert appends, so we're clearing this.
            $message = '';
            $rc = convert_mib($source_file, $dest_file, $message);

            // Error Checking
            $errors == array();

            /** Note: We're ignoring the actual failure to convert ($rc === 1).
             * Due to issues with how snmpttconvertmib parses MIB files, the script will frequently
             * claim to have "unsuccessful translations". Usually, this is because the script has
             * found a keyword ("NOTIFICATION-TYPE" or "TRAP-TYPE", and possibly others) in an 
             * inappropriate location, i.e. in a quoted string, or in the definition of the macro. 
             * In these cases, the rest of the file should still convert correctly, so we're letting it be.
             *
             * If this doesn't make sense to you, try running snmpttconvertmib on RFC-1215.txt and 
             * NOTIFICATION-LOG-MIB.txt, and look for where the errors occur.
             */
            if ($rc === 2) {
                // Don't need to link file, no trap defs found in MIB.
                $success_count += 1;
            }
            else if ($rc === 4) {
                // Unable to find snmpttconvertmib.
                $errors[] = $message;
                break;
            }
            else if (file_exists($dest_file)) {
                if ($nxti_call) {
                    $success_count += 1;
                    $files[] = $dest_file;
                }
                else if (add_mib_to_conf($dest_file, $message) !== 0) {
                    unlink($dest_file);
                    $errors[] = sprintf(_('Unable to add %s to snmptt.ini'), $dest_file);
                }
                else {
                    $success_count += 1;
                }
            }
            else {
                $errors[] = _("Unable to create file ") . $dest_file;
            }
        }
        else {
            $success_count += 1;
        }
    }
    // Cleanup
    closedir($source_directory_fd);

    if ($nxti_call) {
        return $files;
    }

    if ($total_count === $success_count) {
        $result_message = $total_count . _(' MIB files processed') . "<br>";
        $error = false;
    }
    else {
        $result_message = $success_count . _(' of ') . $total_count . ' ' . _("MIB files successfully processed") . '<br>';
        $result_message .= implode("<br/>", $errors);
        $error = true;
    }

    if (restart_snmptt($restart_message) !== 0)
        $result_message .= "<br>" . $restart_message;

    show_mibs($error, $result_message);
}

function do_download()
{

    $file = grab_request_var("download", "");


    // clean the filename
    $file = str_replace("..", "", $file);
    $file = str_replace("/", "", $file);
    $file = str_replace("\\", "", $file);

    $dir = get_mib_dir();
    $thefile = $dir . "/" . $file;

    $mime_type = "";
    header('Content-type: ' . $mime_type);
    header("Content-length: " . filesize($thefile));
    header('Content-Disposition: attachment; filename="' . basename($thefile) . '"');
    readfile($thefile);
    exit();
}

/**
 * @param $src
 * @param $dest
 * @param $message
 *
 * @return int
 */
function convert_mib($src, $dest, &$message)
{
    $rc = 0;
    $lrc = 0;
    $convertmib_location = '/usr/bin/snmpttconvertmib';
    $integration_path = "<a href='https://assets.nagios.com/downloads/nagiosxi/docs/Integrating_SNMP_Traps_With_Nagios_XI.pdf' target='_blank'>here</a>";
    $locate_output = array();
    $convert_results = array();
    $matches = array();
    $success_string = array();

    // locate snmpttconvertmib
    $locate_convertmib = 'which snmpttconvertmib 2>&1';
    exec($locate_convertmib, $locate_output, $lrc);

    // construct and execute command if snmpttconvertmib is available
    if ($lrc != 0) {
        // snmpttconvertmib.py is missing or not found error code
        $rc = 4;
    } else {
        if (!empty($locate_output))    
            $convertmib_location = $locate_output[0];
        $convert_cmd = $convertmib_location . ' --in=' . escapeshellarg($src) . ' --out=' . escapeshellarg($dest) . ' 2>&1';
        exec($convert_cmd, $convert_results, $rc);
    }

    if ($rc != 0) {
        if ($rc == 4) {
            // show location output error and link to integration script
            $message = sprintf(_("snmpttconvertmib is not in the correct location or is not installed. If you have not run the %s SNMP Integration script it is located %s."), get_product_name(), $integration_path) . " ( rc: " . $rc . " = > " . $locate_output[0] . " ) ";
        } else {
            $message = _("Failed to convert uploaded file to snmptt mib.") . " (rc: " . $rc . ")";
        }
    }
    else {
        foreach ($convert_results as $line) {
            if (preg_match('/Failed translations:\s*(\d+)/', $line, $matches) === 1) {
                if ($matches[1] > 0) {
                    $rc = 1;
                    $message = _("Uploaded file had one or more failed translations when converting to snmptt mib.");
                    break;
                }
            } else if (strpos($line, "translations:") !== false) {
                $success_string[] = $line;
            } else if (strpos($line, "MIB file did not contain any TRAP-TYPE or NOTIFICATION-TYPE") !== false) {
                $rc = 2;
                $success_string[] = "MIB is added, but is does not contain any TRAP-TYPE definitions, so no traps were added, even though user input said there should be traps specified.";
                break;
            }
        }

        if ($rc === 0 || $rc === 2) {
            $message = implode("<br />", $success_string);
        }
    }
    return $rc;
}


/**
 * @return string
 */
function get_snmptt_ini_path()
{
    return '/etc/snmp/snmptt.ini';
}


/**
 * @param $mib
 * @param $message
 *
 * @return int
 */
function add_mib_to_conf($mib, &$message)
{
    //add processed mib to snmptt.ini
    $sed_status = 0;
    $sed_results = array();
    $snmptt_ini_path = get_snmptt_ini_path();

    $grep_cmd = "grep $mib $snmptt_ini_path";
    exec($grep_cmd, $grep_arry, $rc);
    if ($rc == 0) {
        return 0; //already in file
    }

    $add_to_snmpttini_cmd = "/bin/sed -i '/^snmptt_conf_files/a" . $mib . "' " . $snmptt_ini_path;
    exec($add_to_snmpttini_cmd, $sed_results, $sed_status);
    if ($sed_status != 0) {
        $message = _("Failed to add converted mib path to snmptt.ini") . ". (rc: " . $sed_status . ")";
    }

    return $sed_status;
}

/**
 * @param $mib
 * @param $message
 *
 * @return mixed
 */
function run_addmib($mib, &$message)
{
    //run addmib command if command exists
    $addmib = '/usr/local/bin/addmib';
    $success_output = array();

    if (is_executable($addmib)) {
        exec($addmib . " " . $mib, $addmib_result, $addmib_returncode);

        if ($addmib_returncode != 0) {
            // send error message from exec
            $message = implode("<br>", $addmib_result);
            return 1;
        } else {
            foreach ($addmib_result as $line) {
                if (strpos($line, "translations:") !== false) {
                    $success_output[] = $line;
                } else if (strpos($line, "MIB file did not contain any TRAP-TYPE or NOTIFICATION-TYPE") !== false) {
                    $success_output[] = "MIB is added, but it does not contain any TRAP-TYPE definitions, so no traps were added, even though user input said there should be traps specified.";
                    break;
                }
            }

            $message = implode("<br>", $success_output);
            return 0;
        }
    }

    // addmib not executable
    return 1;
}


// function restart_snmptt() is now in utils-traps.inc.php


/**
 * @param $processMib
 */
function do_upload($processMib, $import = false) {
    // demo mode
    if (in_demo_mode() == true)
        show_mibs(true, _("Changes are disabled while in demo mode."));
    if ($processMib || $import) {
        if (!file_exists(get_snmptt_ini_path())) {
            show_mibs(true, _('MIB could not be installed - snmptt is not installed.'));
            exit();
        } elseif (!is_writable(get_snmptt_ini_path())) {
            show_mibs(true, 'snmptt.ini ' . _('is not writable by Nagios.') . '<br>' . _('Run the following from the command line') . ':<br><br>chown root.nagios /etc/snmp/snmptt.ini /etc/snmp<br>
    chmod g+w /etc/snmp/snmptt.ini /etc/snmp');
            exit();
        }
    }

    // check session
    check_nagios_session_protector();

    //print_r($request);

    $target_path = get_mib_dir();
    $target_path .= "/";
    $target_path .= basename(str_replace(" ", "_", $_FILES['uploadedfile']['name']));

    $error = $_FILES['uploadedfile']['error'];
    if ($error == UPLOAD_ERR_OK) {
        $test_write = move_uploaded_file($_FILES['uploadedfile']['tmp_name'], $target_path);
    } else if ($error == UPLOAD_ERR_INI_SIZE) {
        $test_write = false;
        $additional_error = _('Your php.ini upload_max_size is likely lower than the size of the uploaded MIB. Update it and restart apache.');
    } else {
        $test_write = false;
        $additional_error = _('Directory permissions may not be correct. Check that /usr/share/snmp/mibs is writable by the nagios group.');
    }

    if (!$test_write) {
        // error
        show_mibs(true, _("MIB could not be installed.") . ' ' . $additional_error);
    } else if ($processMib || $import) {
        chmod($target_path, 0664);
        // success!

        // process mib
        $convert_path = get_processing_destination($import);

        $target_path_parts = pathinfo($target_path);
        $convert_path .= '/' . $target_path_parts['filename'] . '.txt';
        $convert_message = "";

        if ($import) {
            // snmpttmibconvert appends, so we're clearing this.
            if (!unlink($convert_path)) {
            }
            $rc = convert_mib($target_path, $convert_path, $convert_message);
            if ($rc === 4) {
                // Unable to find snmpttconvertmib.
                $message = $convert_message;
            }
            else if (!file_exists($convert_path)) {
                $message = _("Unable to create file ") . $convert_path;
            }
            $result_okay = handle_nxti_import($convert_path, $message);
            show_mibs(!$result_okay, $message);
        }
        else {
            //run addmib if present
            $rc_add = run_addmib($target_path, $convert_message);

            //Process mib with convertmib if addmib isn't executable or isn't installed
            if ($rc_add != 0) {
                $rc = convert_mib($target_path, $convert_path, $convert_message);
                if ($rc !== 0) {
                    if ($rc === 1) {
                        unlink($convert_path);
                    }
                    show_mibs(true, $convert_message);
                } else if (add_mib_to_conf($convert_path, $message) != 0) {
                    unlink($convert_path);
                    show_mibs(true, $message);
                } else {
                    $result_message = _('MIB file successfully processed') . ": <br>";
                    $result_message .= $convert_message;
                    if (restart_snmptt($restart_message) !== 0)
                        $result_message .= "<br>" . $restart_message;

                    show_mibs(false, $result_message);
                }
            }
        }

        $result_message = _('MIB file successfully processed') . ". <br>";
        $error = false;

        if ($convert_message != "") {
            $result_message = $convert_message;
            $error = true;
        }
        if (restart_snmptt($restart_message) !== 0)
            $result_message .= "<br>" . $restart_message;

        show_mibs($error, $result_message);

    } else {
        // successfully uploaded file
        chmod($target_path, 0664);
        show_mibs(false, _("New MIB was installed successfully."));
    }

    exit();
}

function do_delete()
{
    // demo mode
    if (in_demo_mode() == true)
        show_mibs(true, _("Changes are disabled while in demo mode."));

    // check session
    check_nagios_session_protector();

    $file = grab_request_var("delete", "");

    // clean the filename
    $file = str_replace("..", "", $file);
    $file = str_replace("/", "", $file);
    $file = str_replace("\\", "", $file);

    $dir = get_mib_dir();
    $thefile = $dir . "/" . $file;

    if (unlink($thefile) === TRUE) {
        // success!
        show_mibs(false, _("MIB deleted."));
    } else {
        // error
        show_mibs(true, _("MIB delete failed - directory permissions may be incorrect."));
    }
}

// Get the desination directory, create it if it doesn't exist
function get_processing_destination($nxti_call = false) {

    $source_directory = get_mib_dir();
    $dest_directory = $source_directory . "/processed_mibs";
    if ($nxti_call) {
        $dest_directory = "/etc/snmp/tmp";
    }
    if (!is_dir($dest_directory)) {
        mkdir($dest_directory);
    }
    return $dest_directory;
}

?>